#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces

# import packages


class ContextSelector(object):
    """Interface based context selector

    This selector can be used as a subscriber predicate to define
    an interface that the context must support for the event to be applied::

        from pyams_utils.interfaces.site import ISiteRoot

        @subscriber(IObjectModifiedEvent, context_selector=ISiteRoot)
        def siteroot_modified_event_handler(event):
            '''This is an event handler for an ISiteRoot object modification event'''
    """

    def __init__(self, ifaces, config):
        if not isinstance(ifaces, (list, tuple)):
            ifaces = (ifaces,)
        self.interfaces = ifaces

    def text(self):
        return 'context_selector = %s' % str(self.interfaces)

    phash = text

    def __call__(self, event):
        for intf in self.interfaces:
            try:
                if intf.providedBy(event.object):
                    return True
            except (AttributeError, TypeError):
                if isinstance(event.object, intf):
                    return True
        return False
