#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from pyams_utils.interfaces import MissingRequestError
from zope.annotation.interfaces import IAttributeAnnotatable, IAnnotations

# import packages
from pyramid.request import Request
from pyramid.threadlocal import get_current_request, get_current_registry
from zope.interface import alsoProvides


def get_request(raise_exception=True):
    """Get current request

    Raises a NoInteraction exception if there is no active request.
    """
    request = get_current_request()
    if (request is None) and raise_exception:
        raise MissingRequestError("No request")
    return request


def query_request():
    """Query current request

    Returns None if there is no active request"""
    try:
        return get_request()
    except MissingRequestError:
        return None


def check_request(path='/', environ=None, base_url=None, headers=None, POST=None, **kw):
    """Get current request, or create a new blank one if missing"""
    try:
        return get_request()
    except MissingRequestError:
        request = Request.blank(path, environ, base_url, headers, POST, **kw)
        request.registry = get_current_registry()
        return request


def get_annotations(request):
    """Define 'annotations' request property

    This function is automatically defined as a custom request method on package include.
    """
    alsoProvides(request, IAttributeAnnotatable)
    return IAnnotations(request)


def get_debug(request):
    """Define 'debug' request property

    This function is automatically defined as a custom request method on package include.
    """
    class Debug():
        def __init__(self):
            self.showTAL = False
            self.sourceAnnotations = False
    return Debug()


def get_request_data(request, key, default=None):
    """Get data associated with request

    :param request: the request containing requested data
    :param str key: request data annotation key
    :param object default: the default value when data is missing
    :return: the requested value, or *default*
    """
    annotations = request.annotations
    return annotations.get(key, default)


def set_request_data(request, key, value):
    """Associate data with request

    :param request: the request in which to set data
    :param str key: request data annotation key
    :param object value: the value to be set in request annotation
    """
    annotations = request.annotations
    annotations[key] = value
