#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces

# import packages


def get_session_data(request, app, key, default=None):
    """Get data associated with current user session

    PyAMS session management is based on :py:mod:`Beaker` package session management.

    :param request: the request from which session is extracted
    :param str app: application name
    :param str key: session data key for given application
    :param default: object; requested session data, or *default* if it can't be found

    .. code-block:: python

        APPLICATION_KEY = 'MyApp'
        SESSION_KEY = 'MyFunction'

        def my_function(request):
            return get_session_data(request, APPLICATION_KEY, SESSION_KEY)
    """
    session = request.session
    return session.get('{0}::{1}'.format(app, key), default)


def set_session_data(request, app, key, value):
    """Associate data with current user session

    :param request: the request from which session is extracted
    :param str app: application name
    :param str key: session data key for given application
    :param object value: any object that can be pickled can be stored into user session

    .. code-block:: python

        APPLICATION_KEY = 'MyApp'
        SESSION_KEY = 'MyFunction'

        def my_function(request):
            value = {'key1': 'value1', 'key2': 'value2'}
            set_session_data(request, APPLICATION_KEY, SESSION_KEY, value)
    """
    session = request.session
    session['{0}::{1}'.format(app, key)] = value
